package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.platform.bean.SysOrderNoRule;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description : 单号生成规则服务器
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-21 19:57
 * @Modify:
 **/
public interface ISysOrderNoRuleService {

    /**
     * 添加单号规则
     * @param sysOrderNoRule 单号信息
     * @return 新增后的单号信息
     */
    @ApiOperation(value = "添加单号规则")
    SysOrderNoRule insertSysOrderNoRule(SysOrderNoRule sysOrderNoRule);

    /**
     * 修改单号信息
     * @param sysOrderNoRule
     */
    @ApiOperation(value = "修改单号规则")
    void updateSysOrderNoRule(SysOrderNoRule sysOrderNoRule);

    /**
     * 删除单号信息
     * @param id 单号ID
     */
    @ApiOperation(value = "根据id删除单号规则")
    void deleteSysOrderNoRuleById(Long id);

    /**
     * 根据ID 查询单号
     * @param id 单号ID
     * @return 单号信息
     */
    @ApiOperation(value = "根据id查询单号规则")
    SysOrderNoRule getSysOrderNoRuleById(Long id);

    /**
     * 获取分页单号信息
     * @param sysOrderNoRule 单号查询条件
     * @param pager 分页条件
     * @return 分页后的查询集合
     */
    @ApiOperation(value = "单号规则复杂查询，分页，排序")
    ListPager querySysOrderNoRuleByPager(SysOrderNoRule sysOrderNoRule, Pager pager);

    /**
     * 根据单号代码获取单号信息
     * @param code 单号代码
     * @return 单号信息
     */
    @ApiOperation(value = "根据code查询最新单号规则")
    SysOrderNoRule doGetSysOrderNoRuleCode(String code);

    /**
     * 根据id修改单号规则状态
     * @param id 单号ID
     * @param status  单号状态
     */
    @ApiOperation(value = "根据id修改单号规则状态")
    void updateSysOrderNoRuleCodeStatusById(Long id,Integer status);

    /**
     * 批量删除单号信息
     * @param ids 单号ID集合
     */
    @ApiOperation(value = "批量删除单号规则")
    void deleteSysOrderNoRuleByIds(Long[] ids);

    /**
     * 批量修改编号规则状态
     * @param ids 编ID集合
     * @param status 编号状态
     */
    @ApiOperation(value = "批量修改编号规则状态")
    void updateSysOrderNoRuleByIds(Long[] ids,Integer status);

    /**
     * 批量添加编号规则
     * @param sysOrderNoRuleList 编号集合
     */
    @ApiOperation(value = "批量添加编号规则")
    void insertSysOrderNoRuleList(List sysOrderNoRuleList);
}
