package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;

import java.util.List;

/**
 * @Description : 系统资源相关服务，所有sys开头的对象都可在此服务中处理
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-11 16:59
 * @Modify:
 **/
public interface ISystemResourceService {

    /**
     * 新增语言信息
     * @param lang 语言对象
     * @return 新增后的语言信息
     */
    SysLocaleLanguage insertSysLocaleLanguage(SysLocaleLanguage lang);

    /**
     * 修改语言信息
     * @param lang  语言对象
     * @return 修改后的语言对象
     */
    SysLocaleLanguage updateSysLocaleLanguage(SysLocaleLanguage lang);

    /**
     * 删除语言信息
     * @param id
     */
    void deleteSysLocaleLanguageById(long id);

    /**
     * 查询语言信息
     * @param lang  语言信息查询条件
     * @return  语言集合
     */
    List<SysLocaleLanguage> listSysLocaleLanguage(SysLocaleLanguage lang);

    /**
     * 语言信息分页查询
     * @param lang 语言信息查询条件
     * @param pager 分页信息
     * @return 分页后的语言信息
     */
    ListPager<SysLocaleLanguage> listSysLocaleLanguageByPager(SysLocaleLanguage lang, Pager pager);

    /**
     * 新增本地资源
     * @param res 资源信息
     * @return 新增后的资源信息
     */
    SysLocaleResource insertSysLocaleResource(SysLocaleResource res);

    /**
     * 修改资源信息
     * @param res 资源信息
     * @return 修改后的资源信息
     */
    SysLocaleResource updateSysLocaleResource(SysLocaleResource res);

    /**
     * 删除资源信息
     * @param id 资源ID
     */
    void deleteSysLocaleResourceById(long id);

    /**
     * 批量删除资源信息
     * @param ids 资源ID集合
     */
    void deleteSysLocaleResourceByIds(Long[] ids);

    /**
     * 资源信息分页查询
     * @param res  资源信息查询条件
     * @param pager 分页条件
     * @return 资源信息分页集合
     */
    ListPager<SysLocaleResource> listSysLocaleResourceByPager(SysLocaleResource res,Pager pager);

    /**
     * 资源信息查询
     * @param res 资源信息查询条件
     * @return 资源信息集合
     */
    List<SysLocaleResource> listSysLocaleResource(SysLocaleResource res);

    /**
     * 获取单个资源信息
     * @param resourceType 资源类型
     * @param langCode 资源编码
     * @param langKey 资源Key
     * @return 资源信息
     */
    SysLocaleResource getSysLocaleResourceByLanguage(int resourceType,String langCode,String langKey);

    /**
     * 资源信息查询
     * @param res 资源信息查询条件
     * @return 资源信息集合
     */
    Integer getSysLocaleResourceCount(SysLocaleResource res);
}
