package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.MessagePagerModel;
import cn.estsh.i3plus.pojo.model.platform.RefUserMessagePagerModel;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserMessage;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2019-05-06 22:21
 * @Modify:
 **/
@Component
public class BaseMessageController extends CoreBaseController {

    @Autowired
    private ISysMessageService sysMessageService;

    @Autowired
    private ISysUserService sysUserService;

    /*********************************************** 系统消息通用方法 ***********************************************/

    /**
     * 通用 - 系统消息- 新增
     *
     * @param sysMessage 系统消息
     * @return 处理结果
     */
    public ResultBean basaeInsertSysMessage(SysMessage sysMessage) {
        try {
            startMultiService();

            // 条件校验
            ValidatorBean.beginValid(sysMessage)
                    .notNull("messageTitle", sysMessage.getMessageTitle())
                    .notNull("messageContent", sysMessage.getMessageContent())
                    .checkNotZero("messageType", sysMessage.getMessageType())
                    .notNull("messageReceiversId", sysMessage.getMessageReceiversId());

            // 设置发件人名称
            if (sysMessage.getMessageSenderId() == null || sysMessage.getMessageSenderNameRdd() == null) {
                sysMessage.setMessageSenderId(getSessionUser().getUser().getId());
                sysMessage.setMessageSenderNameRdd(getSessionUser().getUserName());
            }

            // 设置收件人名称
            if (sysMessage.getMessageReceiversNameRdd() == null) {
                String[] messageReceiver = sysMessage.getMessageReceiversId().split(",");
                String receiversName = sysUserService.findSysUserByIds(StringTool.getArrayLong(messageReceiver))
                        .stream().map(e -> e.getUserName()).collect(Collectors.joining(","));

                sysMessage.setMessageReceiversNameRdd(receiversName);
            }

            // 设置接收人类型
            if (sysMessage.getMessageReceiverType() == null) {
                sysMessage.setMessageReceiverType(ImppEnumUtil.MESSAGE_RECEIVER_TYPE.INTERNAL.getValue());
            }

            sysMessage.setMessageSendTime(TimeTool.getNowTime(true));
            sysMessage.setMessageContentType(ImppEnumUtil.MESSAGE_TYPE_CONTENT.HTML.getValue());
            sysMessage.setReadCount(0L);

            if(AuthUtil.getSession() != null){
                sysMessage.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            }

            sysMessageService.doSendSysMessage(sysMessage);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 通用 - 系统消息- 根据id删除
     *
     * @param id 系统消息
     * @return 处理结果
     */
    public ResultBean baseDeleteSysMessageById(String id) {
        try {
            // 条件校验
            ValidatorBean.checkNotNull(id, "id不能为空");

            sysMessageService.deleteSysMessageById(Long.parseLong(id));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 通用 - 系统消息- 修改
     *
     * @param sysMessage 系统消息
     * @return 处理结果
     */
    public ResultBean baseUpdateSysMessage(SysMessage sysMessage) {
        try {
            // 条件校验
            ValidatorBean.beginValid(sysMessage)
                    .notNull("messageTitle", sysMessage.getMessageTitle())
                    .notNull("messageContent", sysMessage.getMessageContent())
                    .checkNotZero("messageType", sysMessage.getMessageType());

            sysMessageService.updateSysMessage(sysMessage);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 通用 - 系统消息- 根据id查询
     * @param idStr 消息id
     * @return 处理结果
     */
    public ResultBean<SysMessage> baseGetSysMessageById(String idStr) {
        try {
            SysMessage sysMessage = sysMessageService.getSysMessageById(Long.parseLong(idStr));
            if (sysMessage != null) {
                return ResultBean.success("操作成功").setResultObject(sysMessage).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 通用 - 系统消息 - 分页带条件查询
     * @param messagePagerModel 分页及查询条件信息
     * @return 处理结果
     */
    public ResultBean<SysMessage> baseQuerySysMessageByPager(@RequestBody MessagePagerModel messagePagerModel) {
        try {
            // 数据权限
            if(AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
                messagePagerModel.getSysMessage().setOrganizeCode(null);
            } else {
                messagePagerModel.getSysMessage().setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            }

            ListPager sysMessageListPager = sysMessageService.querySysMessageByPager(messagePagerModel.getSysMessage(),
                    messagePagerModel.getPager());
            return ResultBean.success("操作成功").setListPager(sysMessageListPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 通用 - 系统消息 - 批量删除消息
     *
     * @param ids id数组
     * @return 处理结果
     */
    public ResultBean baseDeleteSysMessageByIds(String[] ids) {
        try {
            // 条件判断
            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            if (ids.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }
            sysMessageService.deleteSysMessageByIds(StringTool.getArrayLong(ids));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /*********************************************** 系统消息通用方法 ***********************************************/

    /**
     * 通用 - 根据用户消息id查询消息
     * @param idStr 用户消息id
     * @return 处理结果
     */
    public ResultBean<SysMessage> baseGetUserMessageById(String idStr) {
        try {
            ValidatorBean.checkNotNull(idStr, "id");

            SysMessage userMessage = sysMessageService.doGetSysMessageByRefUserMessageId(Long.parseLong(idStr));
            ValidatorBean.checkNotNull(userMessage, "消息不存在");
            return ResultBean.success("查询成功").setResultObject(userMessage).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 通用 - 批量切换用户消息状态
     *
     * @param ids    ids数组
     * @param status 状态
     * @return 处理结果
     */
    public ResultBean baseUpdateSysMessageStatusByIds(String[] ids, Integer status) {
        try {
            // 条件判断
            ValidatorBean.checkNotZero(status, "状态不能为空");
            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            if (ids.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            sysMessageService.updateSysRefUserMessageStatusByIds(StringTool.getArrayLong(ids), status);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 通用 - 分页带条件查询用户消息
     *
     * @param refUserMessagePagerModel 查询条件及分页信息
     * @return 处理结果
     */
    public ResultBean<SysRefUserMessage> baseQuerySysRefUserMessageByPager(RefUserMessagePagerModel refUserMessagePagerModel) {
        try {
            // 数据权限
            if(AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
                refUserMessagePagerModel.getSysRefUserMessage().setOrganizeCode(null);
            } else {
                refUserMessagePagerModel.getSysRefUserMessage().setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            }

            ListPager sysMessageListPager = sysMessageService.querySysRefUserMessageByPager(
                    refUserMessagePagerModel.getSysRefUserMessage(), refUserMessagePagerModel.getPager());
            return ResultBean.success("操作成功").setListPager(sysMessageListPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 通用 - 按id删除用户消息
     *
     * @param idStr 用户消息id
     * @return 处理结果
     */
    public ResultBean<SysRefUserMessage> baseDeleteSysRefUserMessageById(String idStr) {
        try {
            sysMessageService.deleteSysRefUserMessageById(Long.parseLong(idStr));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
