package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.ISynchronizedService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginService;
import cn.estsh.i3plus.core.api.iservice.busi.*;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.CookieTool;
import cn.estsh.i3plus.platform.common.tool.ServletRequestTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.UserDetailModel;
import cn.estsh.i3plus.pojo.model.platform.UserDetailPagerModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authc.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2018-12-19 15:31
 * @Modify:
 **/
@RestController
@Api(tags = "非权限服务")
@RequestMapping("/impp/white")
public class WhiteController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(WhiteController.class);

    @Autowired
    private ISysLocaleLanguageService sysLocaleLanguageService;

    @Autowired
    private ISysLocaleResourceService sysLocaleResourceService;

    @Autowired
    private ISystemLoginService systemLoginService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ISystemInitService systemInitService;

    @Autowired
    private ISysFileService sysFileService;

    @Autowired
    private ISysDictionaryService dictionaryService;

    @Autowired
    private ISynchronizedService synchronizedService;

    @Autowired
    private ISysOrganizeService sysOrganizeService;

    @Autowired
    private ISysLabelTemplateService sysLabelTemplateService;

    @Autowired
    private ISysConfigService configService;

    @Resource(name="redisRes")
    private ImppRedis redisRes;

    @GetMapping(value = "/sys-locale-language/list")
    @ApiOperation(value = "查询可用语言",notes = "查询可用语言")
    public ResultBean listSysLocaleLanguage(){
        try {
            List sysLocaleLanguageList = sysLocaleLanguageService.findSysLocaleLanguageByStatus(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(sysLocaleLanguageList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/system-info")
    @ApiOperation(value = "查询可用语言",notes = "查询可用语言")
    public ResultBean getSystemInfo(){
        try {
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(configService.findSystemInfoConfig());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     *
     * @return
     */
    @GetMapping(value="/auth/logout")
    @ApiOperation(value="登出",notes="登出")
    public ResultBean logout(){
        LOGGER.info("用户退出...");
        try {
            AuthUtil.logout();
            return new ResultBean(true,"您已安全退出。").build();
        } catch (ImppBusiException e) {
            return new ResultBean(false,e.getErrorDetail() + "-[解决]" + e.getErrorSolution());
        }
    }

    @GetMapping(value="/auth/login")
    @ApiOperation(value="登陆",notes="登陆")
    public ResultBean login(HttpServletRequest request, String loginName, String loginPwd,
                            @RequestParam(required = false) String languageCode,
                            @RequestParam(required = false) String ipAddr,
                            @RequestParam(required = false) String deviceId){
        Long startTime = System.currentTimeMillis();
        startMultiService();

        LOGGER.info("用户登陆 loginName:{} loginPwd:{} languageCode:{}",loginName,loginPwd,languageCode);
        ResultBean result = null;
        try {
            ValidatorBean.checkNotNull(loginName,"用户名不能为空");
            ValidatorBean.checkNotNull(loginPwd,"密码不能为空");
            ValidatorBean.checkNotNull(languageCode,"语言不能为空");
            logout();

            SessionUser user = userService.queryUserLogin(loginName.trim(), loginPwd.trim(), languageCode, getLoginPlatform().getValue(),deviceId);

            String redisKey = CommonConstWords.USER_LOGIN_ERROR + "_" + user.getUser().getId();
            redisCore.deleteKey(redisKey);

            LOGGER.info("会员登陆：{}",user);

            result = new ResultBean(true, AuthUtil.getSession().getId().toString(), AuthUtil.getSessionUser());
            result.setUrl("/");

            // 记录登录记录
            recordSysUserLog(user.getUser().getUserInfoId(), loginName,CommonEnumUtil.USER_LOGIN_STATUS.LOGIN_SUCCESS.getValue(), ipAddr);
            AuthUtil.setOrganize(user.getUser().getOrganize());
        } catch (IncorrectCredentialsException e) {
            // 密码错误
            Integer num = systemLoginService.doLoginPasswordError(loginName, ipAddr);
            num = CommonConstWords.USER_LOGIN_ERROR_MAX_NUM - num;
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_PASSWORD);
            if(num == 0){
                result.setErrorMsg("密码输入错误。用户已被锁定请联系管理员！");
            }else {
                result.setErrorMsg("密码输入错误。再输入错误"+ num +"次，用户将被锁定。");
            }

            // 记录登录记录
            recordSysUserLog(null, loginName, CommonEnumUtil.USER_LOGIN_STATUS.WRONG_PASSWORD.getValue(), ipAddr);
        } catch (CredentialsException e) {
            // 用户名或密码错误
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_NAME);

            // 记录登录记录
            recordSysUserLog(null, loginName,CommonEnumUtil.USER_LOGIN_STATUS.WRONG_USERNAME_OR_PASSWORD.getValue(), ipAddr);
        } catch (LockedAccountException e) {
            // 账号已锁定
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_LOCKING);

            // 记录登录记录
            recordSysUserLog(null, loginName,CommonEnumUtil.USER_LOGIN_STATUS.USER_LOGIN_LOCKING.getValue(), ipAddr);
        } catch (UnknownAccountException e) {
            // 用户信息不存在
            result = ResultBean.fail(ImppExceptionEnum.LOGIN_EXCEPTION_USER_INFO_NULL);

            // 记录登录记录
            recordSysUserLog(null, loginName,CommonEnumUtil.USER_LOGIN_STATUS.USER_INFO_NULL.getValue(), ipAddr);
        } catch (AuthenticationException e) {
            // 系统异常
            result = ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
            result.setErrorMsg(e.getMessage());

            // 记录登录记录
            recordSysUserLog(null, loginName,CommonEnumUtil.USER_LOGIN_STATUS.SYSTEM_ERROR.getValue(), ipAddr);
        } catch (ImppBusiException e) {
            result = ResultBean.fail(e);

            // 记录登录记录
            recordSysUserLog(null, loginName,CommonEnumUtil.USER_LOGIN_STATUS.SYSTEM_ERROR.getValue(), ipAddr);
        } catch (Exception e) {
            result = ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode())
                    .setErrorMsg(LocaleUtils.getEnumLocaleResDesc(ImppExceptionEnum.SYSTEM_EXCEPTION, ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription()));

            // 记录登录记录
            recordSysUserLog(null, loginName,CommonEnumUtil.USER_LOGIN_STATUS.SYSTEM_ERROR.getValue(), ipAddr);
        }

        LOGGER.info("会员{}登陆登录完成 , 登录耗时:{}",loginName,(System.currentTimeMillis() - startTime));
        return result;
    }

    private void recordSysUserLog(Long userId, String username,Integer userLoginStatus,String IpAddr){
        SysLogUserLogin log = new SysLogUserLogin();

        log.setUserId(userId);
        log.setLogLoginName(username);
        log.setLogLoginStatus(userLoginStatus);
        log.setLogLoginBrowser(ServletRequestTool.getRequestBrowser(request));
        log.setLogLoginHost(IpAddr);
        log.setLogLoginPlatform(CommonEnumUtil.LOG_LOGIN_PLATFORM.descOf(ServletRequestTool.getRequestOs(request)));
        log.setLogLoginDateTime(TimeTool.getNowTime(true));

        userService.insertSysLogUserLogin(log);
    }

    @GetMapping(value = "/find-language-code")
    @ApiOperation(value = "根据语言代码查询资源")
    public ResultBean findSysLocaleResourceByLanguageCode(String languageCode){
        try {
            startMultiService();

            ValidatorBean.checkNotNull(languageCode,"语言代码不能为空");
            if (sysLocaleLanguageService.getSysLocaleLanguageByCode(languageCode) == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("语言不存在")
                        .build();
            }

            // 从缓存获取语言信息
            Map<Object, Object> resourceMap =
                    redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB+ ":" + languageCode);
            if (resourceMap == null || resourceMap.size() == 0) {
                systemInitService.putAndLoadSysLocaleLanguage();
                resourceMap = redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode);
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(resourceMap);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/find-resource")
    @ApiOperation(value = "按条件查询资源")
    public ResultBean<SysLocaleResource> findSysLocaleResource(@RequestBody SysLocaleResource sysLocaleResource){
        try {
            startMultiService();

            List<SysLocaleResource> resourceList = sysLocaleResourceService.findSysLocaleResource(sysLocaleResource);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(resourceList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }



    @GetMapping(value = "/find-language-code-soft-type")
    @ApiOperation(value = "根据语言代码查询资源")
    public ResultBean findSysLocaleResourceByLanguageCode(String languageCode,Integer softType,
                                                          @RequestParam(defaultValue = "2") Integer needPlatformRes){
        try {
            startMultiService();

            ValidatorBean.checkNotNull(languageCode,"语言代码不能为空");
            ValidatorBean.checkNotNull(softType,"软件类型不能为空");
            if (sysLocaleLanguageService.getSysLocaleLanguageByCode(languageCode) == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("语言不存在")
                        .build();
            }

            // 从缓存获取语言信息
            Map<Object, Object> resourceMap =
                    redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode + ":" + softType);
            if (resourceMap == null || resourceMap.size() == 0) {
                systemInitService.putAndLoadSysLocaleLanguage();
                resourceMap = redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode + ":" + softType);
            }

            if(needPlatformRes.intValue() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()){
                Map<Object, Object> platformResMap =
                        redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + languageCode + ":" + CommonEnumUtil.SOFT_TYPE.IMPP.getValue());;
                resourceMap.putAll(platformResMap);
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(resourceMap);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @PostMapping(value = "/sys-file/insert")
    @ApiOperation(value = "新增文件资源", notes = "新增文件资源")
    public ResultBean insertSysFile(@RequestBody SysFile sysFile){
        try {
            startMultiService();

            // 条件验证
            ValidatorBean.beginValid(sysFile)
                    .notNull("dfsFileName", sysFile.getDfsFileName())
                    .notNull("dfsGroupName", sysFile.getDfsGroupName())
                    .notNull("fileSoftType", sysFile.getFileSoftType())
                    .notNull("fileOriginalName", sysFile.getFileOriginalName());

            // 文件类型验证
            SysDictionary dictionary = dictionaryService.getSysDictionaryByParentCodeAndValue(BaseConstWords.DICTIONARY_FILE_TYPE,
                    StringTool.getStringFileSuffix(sysFile.getFileOriginalName(), true));
            if(dictionary == null){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_FORMAT.getCode())
                        .setErrorDetail("文件格式不支持 %s",sysFile.getFileOriginalName())
                        .setErrorSolution("请到字典管理中添加文件类型")
                        .build();
            }

            // 初始化数据
            sysFile.setFileTypeId(dictionary.getId());
            sysFile.setFileTypeName(dictionary.getName());

            sysFile = sysFileService.insertSysFile(sysFile);
            return  ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysFile);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据文件代码查询文件信息
     * @param fileCode 文件代码
     * @return 处理结果
     */
    @GetMapping("/sys-file/get-file-code/{code}")
    @ApiOperation(value = "根据文件代码查询文件信息")
    public ResultBean getSysFileByFileCode(@PathVariable("code") String fileCode) {
        try {
            // 条件验证
            ValidatorBean.checkNotNull(fileCode, "文件代码不能为空");
            SysFile sysFile = sysFileService.getSysFileByFileCode(fileCode);

            return  ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysFile);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @GetMapping(value = "/sys-order-no-rule/get-order-no/{code}")
    @ApiOperation(value = "根据单号规则代码，生成单号")
    public ResultBean getOrderNo(@PathVariable("code") String code) {
        try {
            ValidatorBean.checkNotNull(code, "code不能为空");

            SysOrderNoRule sysOrderNoRule = synchronizedService.nextOrderNo(code);
            return ResultBean.success("操作成功").setResultObject(sysOrderNoRule).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/sys-find-user/list")
    @ApiOperation(value = "查询用户列表信息",notes = "查询用户列表信息")
    public ResultBean findSysUserInfoList(UserDetailModel model){
        try {
            List<SysUser> userList = personnelService.querySysUserByUserDetailModelOrIds(model, new ArrayList<>());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(userList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * @return 处理结果
     */
    @GetMapping("/sys-login-platform")
    @ApiOperation(value = "登录标识", notes = "登录标识")
    public ResultBean getSysloginPlatforml() {
        return new ResultBean(true, "操作成功",
                Arrays.asList(CommonEnumUtil.LOG_LOGIN_PLATFORM.values()));
    }

    /**
     * 获取登录平台信息
     * @return
     */
    private CommonEnumUtil.LOG_LOGIN_PLATFORM getLoginPlatform(){
        String value = CookieTool.getCookieValue(request, CommonConstWords.SESSION_LOGIN_PLATFORM);
        if(StringUtils.isBlank(value)){
            value = CommonEnumUtil.LOG_LOGIN_PLATFORM.PC_IMPP_CORE.getValue() + "";
        }else{
            try {
                Integer.parseInt(value);
            }catch (Exception e){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_FORMAT.getCode())
                        .setErrorDetail("平台登录标识转换错误")
                        .setErrorSolution("请重新操作")
                        .build();
            }
        }
        CommonEnumUtil.LOG_LOGIN_PLATFORM platform = CommonEnumUtil.LOG_LOGIN_PLATFORM.valueOf(Integer.parseInt(value));
        return platform == null ? CommonEnumUtil.LOG_LOGIN_PLATFORM.PC_IMPP_CORE : platform;
    }

    @GetMapping("/sys-organize/list")
    @ApiOperation(value = "获取全部组织", notes = "获取全部组织")
    public ResultBean<SysOrganize> findSysOrganizeList(){
        try {
            List<SysOrganize> sysOrganizeList = sysOrganizeService.findSysOrganizeList();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(sysOrganizeList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询全部打印模板
     * @return 处理结果
     */
    @PostMapping(value = "/sys-label-template/list")
    @ApiOperation(value = "查询全部打印模板",notes = "查询全部打印模板")
    public ResultBean<SysLabelTemplate> findSysConfigAll(@RequestBody SysLabelTemplate template){
        try {
            List<SysLabelTemplate> sysLabelTemplateList = sysLabelTemplateService.listSysLabelTemplateDetail(template);;
            return  ResultBean.success("操作成功")
                    .setResultList(sysLabelTemplateList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询未翻译的资源
     * @return 处理结果
     */
    @PostMapping(value = "/locale-resource/no-resource")
    @ApiOperation(value = "查询未翻译的资源",notes = "查询未翻译的资源")
    public ResultBean<SysLabelTemplate> findSysConfigAll(){
        try {
            Set<String> keys = redisRes.getKeysSet(CommonConstWords.REDIS_PREFIX_CACHE_NO_RES+"*");
            List<String> list = new ArrayList<>();
            for (String key : keys) {
                list.add(redisRes.getObject(key).toString());
            }
            return  ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据条件分页查询用户信息
     * @param model 查询条件
     * @return 处理结果
     */
    @PostMapping(value = "/personnel/pager/get-user-list")
    @ApiOperation(value = "根据条件分页查询账号信息", notes = "根据条件分页查询账号信息")
    public ResultBean queryPagerByUser(@RequestBody UserDetailPagerModel model) {
        try {
            ListPager<SysUser> listPager = personnelService.querySysUser(model.getUserDetailModel(), model.getPager());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(listPager);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }
}
