package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysFileService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleResourceService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.BaseImppException;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.FileContentTypeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysFile;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.fastdfs.FastDFSClient;
import cn.estsh.impp.framework.boot.fastdfs.FastDFSFile;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.ListUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.*;

/**
 * @Description :系统语言资源服务
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-12 16:34
 * @Modify:
 **/
@RestController
@Api(tags = "系统语言资源服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-locale-resource")
public class SysLocaleResourceController extends CoreBaseController {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysLocaleResourceController.class);

    @Resource(name=CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @Autowired
    private ISysLocaleResourceService sysLocaleResourceService;

    @Autowired
    private ISystemInitService systemInitService;;

    @Autowired
    private ISysFileService sysFileService;

    @Autowired
    private EntityManager entityManager;

    @Autowired
    private FastDFSClient dfsClient;

    /**
     * 新增语言资源
     * @param sysLocaleResource 语言资源
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增语言资源",notes = "新增语言资源")
    public ResultBean insertSysLocaleResource(SysLocaleResource sysLocaleResource){
        try {
            //条件验证
            ValidatorBean.beginValid(sysLocaleResource)
                    .notNull("resourceKey",sysLocaleResource.getResourceKey())
                    .notNull("languageCode",sysLocaleResource.getLanguageCode())
                    .checkNotZero("resourceType",sysLocaleResource.getResourceType())
                    .notNull("resourceValue",sysLocaleResource.getResourceValue());

            //新增初始化
            sysLocaleResource.setIsSystem(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

            sysLocaleResourceService.insertSysLocaleResource(sysLocaleResource);

            // 更新缓存信息
            updateResourceCache(sysLocaleResource,false);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改语言资源
     * @param sysLocaleResource 修改语言资源
     * @return 处理结果
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改语言资源",notes = "修改语言资源")
    public ResultBean updateSysLocaleResource(SysLocaleResource sysLocaleResource){
        try {
            //条件验证
            ValidatorBean.beginValid(sysLocaleResource)
                    .checkNotZero("id",sysLocaleResource.getId())
                    .notNull("resourceKey",sysLocaleResource.getResourceKey())
                    .notNull("languageCode",sysLocaleResource.getLanguageCode())
                    .checkNotZero("resourceType",sysLocaleResource.getResourceType())
                    .notNull("resourceValue",sysLocaleResource.getResourceValue());

            sysLocaleResourceService.updateSysLocaleResource(sysLocaleResource);

            // 更新缓存信息
            updateResourceCache(sysLocaleResource,false);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id删除系统语言资源
     * @param idStr 语言资源id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "根据id删除系统语言资源",notes = "根据id删除系统语言资源")
    public ResultBean deleteSysLocaleResourceById(@PathVariable("id") String idStr){
        try {
            startMultiService();

            //条件验证
            ValidatorBean.checkNotNull(idStr,"id不能为空");
            SysLocaleResource sysLocaleResource =sysLocaleResourceService.getSysLocaleResourceById(Long.parseLong(idStr));
            if(sysLocaleResource == null){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("资源不存在")
                        .build();
            }

            sysLocaleResourceService.deleteSysLocaleResourceById(Long.parseLong(idStr));

            updateResourceCache(sysLocaleResource,false);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询全部语言资源
     * @return 处理结果
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询全部语言资源",notes = "查询全部语言资源")
    public ResultBean listSysLocaleResource(){
        try {
            List sysLocaleLanguageList = sysLocaleResourceService.listSysLocaleResource();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(sysLocaleLanguageList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 语言资源复杂查询，分页，排序
     * @param sysLocaleResource 查询资源
     * @param pager 分页数据
     * @return 处理结果
     */
    @GetMapping(value = "/query")
    @ApiOperation(value = "语言资源复杂查询，分页，排序",notes = "语言资源复杂查询，分页，排序")
    public ResultBean querySysLocaleResourceByPager(SysLocaleResource sysLocaleResource, Pager pager){
        try {
            ListPager sysLocaleLanguageListPager = sysLocaleResourceService.querySysLocaleResourceByPager(sysLocaleResource,pager);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(sysLocaleLanguageListPager);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 批量删除系统语言资源
     * @param idsStr 语言资源id
     * @return 处理结果
     */
    @DeleteMapping(value = "/batch-delete")
    @ApiOperation(value = "批量删除系统语言资源",notes = "批量删除系统语言资源")
    public ResultBean deleteSysLocaleResourceByIds(String[] idsStr){
        try {
            if(idsStr.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }

            sysLocaleResourceService.deleteSysLocaleResourceByIds(StringTool.getArrayLong(idsStr));

            // 重新加载资源
            systemInitService.putAndLoadSysLocaleLanguage();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 批量添加系统资源
     * @param sysLocaleResourcesList 系统资源集合
     * @return 处理结果
     */
    @PutMapping(value = "/batch-insert")
    @ApiOperation(value = "批量添加系统资源")
    public ResultBean insertSysLocaleResourceList(@RequestBody List<SysLocaleResource> sysLocaleResourcesList){
        try {
            if(sysLocaleResourcesList.size() == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }

            for (SysLocaleResource sysLocaleResource : sysLocaleResourcesList) {
                ConvertBean.modelInitialize(sysLocaleResource,AuthUtil.getSessionUser());
            }

            sysLocaleResourceService.insertSysLocaleResourceList(sysLocaleResourcesList);

            // 重新加载资源
            systemInitService.putAndLoadSysLocaleLanguage();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 更新缓存信息
     * @param sysLocaleResource 系统资源消息
     * @param isDelete 是否删除
     */
    private void updateResourceCache(SysLocaleResource sysLocaleResource,boolean isDelete){
        // 判断是否为删除资源
        if(isDelete){
            Map<Object, Object> langMap = redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + sysLocaleResource.getLanguageCode());
            if (langMap != null && langMap.size() != 0) {
                langMap.remove(sysLocaleResource.getResourceKey());
                redisRes.putHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + sysLocaleResource.getLanguageCode(), langMap);
            }

            redisRes.deleteKey(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + sysLocaleResource.getResourceKey());
        }else {
            Map<Object, Object> langMap = redisRes.getHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + sysLocaleResource.getLanguageCode());
            if (langMap != null && langMap.size() != 0) {
                langMap.put(sysLocaleResource.getResourceKey(), sysLocaleResource.getResourceValue());
                redisRes.putHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + sysLocaleResource.getLanguageCode(), langMap);
            }

            HashMap<String, String> resource = new HashMap<String, String>();
            resource.put(sysLocaleResource.getLanguageCode(), sysLocaleResource.getResourceValue());
            redisRes.putHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + sysLocaleResource.getResourceKey(), resource, 0);
        }
    }

    @PostMapping(value = "/import")
    @ApiOperation(value = "导入系统资源")
    public ResultBean importSysLocaleResource(@RequestParam("file") MultipartFile file){
        try {
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            List<SysLocaleResource> sysLocaleResourceList = excelTool.importData(file.getOriginalFilename(), file.getInputStream(),
                    SysLocaleResource.class);

            // 初始化数据,资源去重
            Map<String, SysLocaleResource> itemMap = new HashMap<>();
            String resourceKey;
            String userName = AuthUtil.getSessionUser().getUserName();
            for (SysLocaleResource localeResource : sysLocaleResourceList) {
                localeResource.setIsSystem(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelInitialize(localeResource, userName);

                resourceKey = localeResource.getResourceKey() + localeResource.getLanguageCode();

                itemMap.put(resourceKey, localeResource);
            }

            List<SysLocaleResource> importDataList = new ArrayList<>(itemMap.values());
            sysLocaleResourceService.deleteSysLocaleResourceAll();

            for (List<SysLocaleResource> resourceList : ListUtils.partition(importDataList, 1000)) {
                new Thread(){
                    @Override
                    public void run() {
                        super.run();
                        sysLocaleResourceService.saveSysLocaleResource(resourceList);
                    }
                }.start();
            }

            // 重新加载资源
//            systemInitService.putAndLoadSysLocaleLanguage();
            return ResultBean.success("导入成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (BaseImppException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/export")
    @ApiOperation(value = "导出系统资源")
    public ResultBean exportSysLocaleResource(HttpServletResponse response)  {

        try {
            startMultiService();
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "locale_" + System.currentTimeMillis() + ".xls";
            List<SysLocaleResource> list = sysLocaleResourceService.listSysLocaleResource();

            File file = new File(fileName);
            file.createNewFile();
            File excle = excelTool.exportData(file, list, SysLocaleResource.class, new String[]{
                    "resourceType", "languageCode", "languageNameRdd", "resourceKey", "resourceValue", "softType"
            });

            String userAgent = request.getHeader("User-Agent");
            // 针对IE或者以IE为内核的浏览器：
            if (userAgent.contains("MSIE") || userAgent.contains("Trident")) {
                fileName = java.net.URLEncoder.encode(fileName, "UTF-8");
            } else {
                // 非IE浏览器的处理：
                fileName = new String(fileName.getBytes("UTF-8"), "ISO-8859-1");
            }

            InputStream targetStream = new DataInputStream(new FileInputStream(excle));
            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(".xls")); // 设置文件名

            BufferedInputStream bis = null;
            try {
                bis = new BufferedInputStream(targetStream);
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (bis != null) {
                    try {
                        bis.close();
                        targetStream.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
            file.delete(); // 删除临时文件
            return ResultBean.success("下载").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/increment-import")
    @ApiOperation(value = "增量导入系统资源")
    public ResultBean incrementImportSysLocaleResource(@RequestParam("file") MultipartFile file){
        try {
            // 重新加载资源
            systemInitService.putAndLoadSysLocaleLanguage();

            Long startTime = System.currentTimeMillis();
            // 提取Excel数据
            LOGGER.info("开始增量导入 {}",file.getOriginalFilename());
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            List<SysLocaleResource> sysLocaleResourceList = excelTool.importData(file.getOriginalFilename(), file.getInputStream(),
                    SysLocaleResource.class);

            long excelToolExecuteTime = System.currentTimeMillis();
            LOGGER.info("增量导入 数据量 {}, ExcelTool耗时 {}", sysLocaleResourceList.size(), excelToolExecuteTime - startTime);

            // 数据去重并初始化

            // 初始化数据,资源去重
            Map<String, SysLocaleResource> itemMap = new HashMap<>();
            String resourceKey;
            String userName = AuthUtil.getSessionUser().getUserName();
            for (SysLocaleResource localeResource : sysLocaleResourceList) {
                // 初始化
                localeResource.setId(getResourceKey(localeResource.getResourceKey(),localeResource.getLanguageCode()));
                localeResource.setIsSystem(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.serviceModelInitialize(localeResource, userName);

                resourceKey = localeResource.getResourceKey() + localeResource.getLanguageCode();

                itemMap.put(resourceKey, localeResource);
            }

            List<SysLocaleResource> importDataList = new ArrayList<>(itemMap.values());

            long deduplicationExecuteTime = System.currentTimeMillis();
            LOGGER.info("数据去重完毕, 数据去重耗时 {}", deduplicationExecuteTime - excelToolExecuteTime);

            // 分批导入
            int size = 1000;
            int i = 0;
            for (List<SysLocaleResource> resourceList : ListUtils.partition(importDataList, size)) {
                new Thread(){
                    @Override
                    public void run() {
                        super.run();
                        long startTime = System.currentTimeMillis();
                        sysLocaleResourceService.saveSysLocaleResource(resourceList);
                        LOGGER.info("增量导入完毕, 导入耗时{}",System.currentTimeMillis() - startTime);
                    }
                }.start();
            }

//            LOGGER.info("数据缓存更新完毕,总耗时：{}",System.currentTimeMillis()-startTime);
            return ResultBean.success("数据导入中...").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (BaseImppException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/down-template")
    @ApiOperation(value = "下载导入模板")
    public ResultBean downSysLocaleResourceTemplate() {
        try {
            startMultiService();
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String importTemplateCode = SysLocaleResource.class.getSimpleName() + "ImportTemplate";

            // 查询服务器中是否存在模板文件
            SysFile sysFile = sysFileService.getSysFileByFileCode(importTemplateCode);
            if(sysFile == null){
                FastDFSFile fastDFSFile = new FastDFSFile(SysLocaleResource.class.getSimpleName() + "ImportTemplate.xls",
                        excelTool.importTemplate(SysLocaleResource.class), ".xls");

                sysFile = dfsClient.upload(fastDFSFile,CommonEnumUtil.SOFT_TYPE.CORE.getValue());
                sysFile.setFileCode(importTemplateCode);
                sysFileService.insertSysFile(sysFile);
            }

            return ResultBean.success("导出成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(sysFile);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/update-cache")
    @ApiOperation(value = "更新资源缓存")
    public ResultBean updateResourceCache(){
        try {
            // 重新加载资源
            systemInitService.putAndLoadSysLocaleLanguage();

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }

    }

    private Long getResourceKey(String resKey, String languageCode) {
        if (redisRes.checkKey(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + resKey, languageCode)) {
            LOGGER.info("查询到已存在资源 {}：{}",resKey,languageCode);
            SysLocaleResource sysLocaleResource =sysLocaleResourceService.getSysLocaleResourceByLanguageCodeAndResKey(
                    languageCode, resKey
            );
            return sysLocaleResource == null ? null : sysLocaleResource.getId();
        }
        return null;
    }
}
