package cn.estsh.i3plus.core.apiservice.controller.report;

import cn.estsh.i3plus.core.api.iservice.base.IReportService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysRoleService;
import cn.estsh.i3plus.platform.common.tool.ClassTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.common.ClassFieldModel;
import cn.estsh.i3plus.pojo.model.common.ClassModel;
import cn.estsh.i3plus.pojo.model.report.TemplateModel;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysRole;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 报表服务
 */
@RestController
@RequestMapping(PlatformConstWords.BASE_URL + "/cloud/report")
@Api(tags = "报表服务")
public class CoreReportController extends CoreBaseController {

    private static final Logger LOGGER = LoggerFactory.getLogger(CoreReportController.class);

    @Value("${impp.app.pojo-packages}.bean")
    private String pojoPackagePath;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private IReportService reportService;

    /**
     * 查询所有角色
     *
     * @return 处理结果
     */
    @GetMapping(value = "/role/list")
    @ApiOperation(value = "查询所有角色", notes = "查询所有系统角色")
    public ResultBean findRoleAll() {
        try {
            List<SysRole> list = roleService.findSysRoleAll();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有角色
     *
     * @param ids 角色id数组
     * @return 处理结果
     */
    @PostMapping(value = "/role/list/ids")
    @ApiOperation(value = "查询所有角色", notes = "查询所有系统角色")
    public ResultBean findRoleByIdList(@RequestBody String[] ids) {
        try {
            List<SysRole> list = roleService.findSysRoleByInId(StringTool.getArrayLong(ids));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询登录用户所有角色
     * @return 处理结果
     */
    @GetMapping(value = "/role/user/list")
    @ApiOperation(value = "查询登录用户所有角色")
    public ResultBean findSysRoleByUser() {
        try {
            SessionUser sessionUser = getSessionUser();
            if (sessionUser != null && sessionUser.getUser() != null) {
                List<SysRole> list = roleService.findSysRoleByUser(getSessionUser().getUser());
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
            }
            return ResultBean.fail("操作失败").setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode()).setErrorMsg("获取用户信息失败");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据核心包路径列出所有类
     * @param model 模板模型
     * @return 处理结果
     */
    @PostMapping(value = "/template-model/run")
    @ApiOperation(value = "查询管理后台所有类", notes = "根据核心包路径列出所有类")
    public ResultBean runTemplateModel(@RequestBody TemplateModel model) {
        LOGGER.info("【执行TemplateModel】TemplateModel：{}", model);
        try {
            List list = reportService.selectTemplateModel(model);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (Exception e) {
            return ResultBean.fail("操作失败").setMsg(e.getMessage()).setErrorMsg(e.getLocalizedMessage()
            );
        }
    }

    /**
     * 查询管理后台所有类
     * @return 处理结果
     */
    @GetMapping(value = "/clz-model/list")
    @ApiOperation(value = "查询管理后台所有类", notes = "根据核心包路径列出所有类")
    public BaseResultBean<ClassModel> listClzModel() {
        LOGGER.info("【类路径：{}】", pojoPackagePath);
        List<String> clzNameList = ClassTool.getClassName(pojoPackagePath, true);
        LOGGER.info("【类信息：{}】", clzNameList);
        List<ClassModel> clzList = new ArrayList<>(clzNameList.size());
        for (String clzName : clzNameList) {
            ClassModel model = ClassTool.getClassModel(clzName);
        }
        return ResultBean.success("获取类：" + clzList.size()).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(clzList);
    }

    /**
     * 查询管理后台所有类
     * @param classPath 类路径
     * @return 处理结果
     */
    @GetMapping(value = "/clz-model/{classPath}")
    @ApiOperation(value = "查询管理后台所有类", notes = "根据核心包路径列出所有类")
    public BaseResultBean<ClassModel> getClzModel(@PathVariable("classPath") String classPath) {
        LOGGER.info("【类路径：{}】", pojoPackagePath);
        ClassModel model = ClassTool.getClassModel(classPath);
        return ResultBean.success("获取类：" + model.getClzDesc()).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(model);
    }

    /**
     * 查询管理后台所有类
     * @return 处理结果
     */
    @GetMapping(value = "/list-clz")
    @ApiOperation(value = "查询管理后台所有类", notes = "根据核心包路径列出所有类")
    public BaseResultBean<ClassModel> listClz() {
        LOGGER.info("【类路径：{}】", pojoPackagePath);
        List<String> clzNameList = ClassTool.getClassName(pojoPackagePath, true);
        List<ClassModel> clzList = new ArrayList<>(clzNameList.size());
        for (String clzName : clzNameList) {
            clzList.add(ClassTool.getClassModel(clzName));
        }

        return ResultBean.success("获取类：" + clzList.size())
                .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                .setResultList(clzList);
    }

    /**
     * 查询类所有属性
     * @param clzFullName 全类名
     * @return 处理结果
     */
    @GetMapping(value = "/pojo-property")
    @ApiOperation(value = "查询类所有属性")
    public ResultBean<ClassFieldModel> testWms(@RequestParam("clzFullName") String clzFullName) {
        LOGGER.info("【查询类属性：{}】", clzFullName);
        List<ClassFieldModel> fieldModelList = ClassTool.getClassFieldModel(clzFullName);
        return ResultBean.success("获取属性：" + fieldModelList.size())
                .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                .setResultList(fieldModelList);
    }



}
