package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.icloud.wms.sdk.IWmsDemoCloud;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.QUEUE_IMPP_MESSAGE;

/**
 * @Description : 获取队列信息后处理
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-09-26 10:40
 * @Modify:
 **/
@Component
public class I3CoreQueueReceiver {
    private static final Logger LOGGER = LoggerFactory.getLogger(I3CoreQueueReceiver.class);

    @Autowired
    IWmsDemoCloud iWmsDemoCloud;

    /**
     * 系统邮件处理队列
     * @param msg 消息
     * @param channel 通道
     * @param message 消息
     * 发送：rabbitTemplate.convertAndSend(I3CoreQueueConfig.QUEUE_IMPP_MESSAGE, new SysMessage(....));
     */
    @RabbitListener(queues = QUEUE_IMPP_MESSAGE)
    public void processImppMessage(SysMessage msg, Channel channel, Message message) {
        try {
            LOGGER.debug("【MQ-QUEUE_IMPP_MESSAGE】数据接收成功：{}",msg);

            //信息已处理
            channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
        } catch (IOException e) {
            LOGGER.error("【MQ-QUEUE_IMPP_MESSAGE】处理出错：{}",e.getMessage(),e);
            //丢弃这条消息
            try {
                // 未成功处理，重新发送
                channel.basicNack(message.getMessageProperties().getDeliveryTag(),false,true);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }

}
