package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleLanguageService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleResourceService;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;
import com.rabbitmq.client.Channel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.QUEUE_IMPP_RESOURCE;


/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 19-8-23 下午3:12
 * @Modify:
 **/
//@Component
public class SysLocalResourceQueue {

    private static final Logger LOGGER = LoggerFactory.getLogger(SysLocalResourceQueue.class);

    @Autowired
    private ISysLocaleResourceService localeResourceService;

    @Autowired
    private ISysLocaleLanguageService localeLanguageService;

    @Autowired
    private ISystemInitService systemInitService;

    /**
     * 语言反向国际化实现
     * <per>
     * <br/>   根据信息创建不同语言的国际化信息。
     * <br/>        当存在的时候不做任何操作，不存的时候创建全部语言的资源信息
     * </per>
     * @param resourceKey
     * @param channel
     * @param message
     */
    @RabbitListener(queues = QUEUE_IMPP_RESOURCE)
    public void processImppMessage(String resourceKey, Channel channel, Message message) {
        try {
            if(StringUtils.isNotBlank(resourceKey)){
                List<SysLocaleLanguage> list = localeLanguageService.listSysLocaleLanguage();
                if(list != null && list.size() > 0){
                    List<SysLocaleResource> resourceList = localeResourceService.findSysLocaleResourceByResKey(resourceKey);

                    List<SysLocaleResource> saveResource = new ArrayList<>(list.size());
                    SysLocaleResource resource = null;
                    for (SysLocaleLanguage language : list) {
                        resource = null;
                        if(resourceList != null && resourceList.size() > 0){
                            for (SysLocaleResource localeResource : resourceList) {
                                if(language.getLanguageCode() != null && language.getLanguageCode().equals(localeResource.getLanguageCode())){
                                    resource = localeResource;
                                }
                            }
                        }else {
                            resource = new SysLocaleResource();
                            resource.setLanguageCode(language.getLanguageCode());
                            resource.setLanguageNameRdd(language.getLanguageName());
                            resource.setIsSystem(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                            resource.setResourceType(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.MISSING.getValue());
                            resource.setResourceValue(resourceKey);
                            resource.setResourceKey(resourceKey);

                            resource.setCreateDatetime(TimeTool.getNowTime(true));
                            resource.setModifyDatetime(TimeTool.getNowTime(true));
                            resource.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());  //有效
                            resource.setIsDeleted(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());   //未删除
                        }

                        if(resource != null){
                            saveResource.add(resource);
                        }
                    }

                    if(saveResource != null && saveResource.size() > 0){
                        localeResourceService.saveSysLocaleResource(saveResource);
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                LOGGER.info("[MQ ACK] Tag :{} Completed successfully",message.getMessageProperties().getDeliveryTag());
            } catch (IOException e) {
                LOGGER.error("[MQ ERROR] MQ Ack Error Message:{}",e.getMessage());
                e.printStackTrace();
            }
        }
    }

}
