package cn.estsh.i3plus.core.apiservice.schedulejob;

import cn.estsh.i3plus.core.api.iservice.busi.ISysDashboardService;
import cn.estsh.i3plus.core.apiservice.controller.busi.SysDashboardController;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.impp.framework.base.schedule.BaseImppScheduleJob;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import org.apache.shiro.web.session.mgt.DefaultWebSessionManager;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description : 仪表盘数据查询
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2019-11-08 15:15
 * @Modify:
 **/
@DisallowConcurrentExecution
public class SysDashboardDataJob extends BaseImppScheduleJob {

    private static final Logger LOGGER = LoggerFactory.getLogger(SysDashboardDataJob.class);

    public SysDashboardDataJob(Class cls, String jobDesc) {
        super(SysDashboardDataJob.class, "仪表盘数据查询");
    }

    @Autowired
    private ISysDashboardService sysDashboardService;

    @Resource(name = CommonConstWords.IMPP_REDIS_CORE)
    private ImppRedis redisCore;

    @Resource(name = "imppSessionManager")
    private DefaultWebSessionManager webSessionManager;

    @Override
    public void executeImppJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date nowDate = new Date();

        // 更新基础信息 获取基础数据，用户，部门，组织等信息
        Map<Object, Object> dashboardData = new HashMap<>();
        dashboardData.put("orgNum", sysDashboardService.getSysOrganizeCount());
        dashboardData.put("deptNum", sysDashboardService.getSysDepartmentCount());
        dashboardData.put("onlineUser", webSessionManager.getSessionDAO().getActiveSessions().size());
        dashboardData.put("userLoginLog", sysDashboardService.findSysUserLoginLog(10));

        LOGGER.debug("获取仪表板基础信息 {}：{}", PlatformConstWords.REDIS_DASHBOARD_BASIC_INFO, dashboardData);
        redisCore.putHashMap(PlatformConstWords.REDIS_DASHBOARD_BASIC_INFO, dashboardData, SysDashboardController.TIMEOUT);

        // 最近操作，异常日志
        dashboardData = new HashMap<>();
        dashboardData.put("logOperate", sysDashboardService.findNewSysLogOperateSize(10));
        dashboardData.put("logException", sysDashboardService.findNewSysLogExceptionSize(10));

        LOGGER.info("最近操作，异常日志 {}：{}", PlatformConstWords.REDIS_DASHBOARD_LOG_INFO, dashboardData);
        redisCore.putHashMap(PlatformConstWords.REDIS_DASHBOARD_LOG_INFO, dashboardData, SysDashboardController.TIMEOUT);

        // 获取平均响应时间,前3分钟时间
        dashboardData = sysDashboardService.querySysLogSystemAvgExecuteTime(
                sdf.format(TimeTool.timeCalc(nowDate, Calendar.MINUTE, -3)), TimeTool.getNowTime(true));

        LOGGER.info("获取平均响应时间 {}：{}", PlatformConstWords.REDIS_DASHBOARD_AVG_RESP_TIME, dashboardData);
        redisCore.putHashMap(PlatformConstWords.REDIS_DASHBOARD_AVG_RESP_TIME, dashboardData, SysDashboardController.TIMEOUT);

        // 获取各等级日志数量，前10分钟的日志数量
        dashboardData = sysDashboardService.querySysLogSystemByLevel(
                sdf.format(TimeTool.timeCalc(nowDate, Calendar.MINUTE, -10)), TimeTool.getNowTime(true));

        LOGGER.info("获取各等级日志数量 {}：{}", PlatformConstWords.REDIS_DASHBOARD_LOG_NUM, dashboardData);
        redisCore.putHashMap(PlatformConstWords.REDIS_DASHBOARD_LOG_NUM, dashboardData, SysDashboardController.TIMEOUT);


        // 获取服务压力情况
        dashboardData = new HashMap<>();
        //获取上一分钟,请求来源数据
        Map<?, ?> dataMap = redisCore.getHashMap(
                CommonConstWords.REDIS_DASHBOARD_MINUTE_SOURCE + "_" + sdf.format(TimeTool.timeCalc(nowDate, Calendar.MINUTE, -1)));
        if (dataMap == null) {
            dataMap = new HashMap<>();
        }
        dashboardData.put("requestSource", dataMap);

        //获取上一秒，服务器压力数据
        sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        Object pressureObj = redisCore.getObject(
                CommonConstWords.REDIS_DASHBOARD_SECOND_PRESSURE + "_" + sdf.format(TimeTool.timeCalc(nowDate, Calendar.SECOND, -1)));
        int pressure = 0;
        if (pressureObj != null) {
            try {
                pressure = Integer.parseInt(pressureObj.toString());
            } catch (Exception e) {
                LOGGER.error("【无法转换压力】data：{}", pressureObj);
            }
        }
        dashboardData.put("serverPressure", pressure);

        LOGGER.info("保存服务压力情况 {}：{}", PlatformConstWords.REDIS_DASHBOARD_SERVER_PRESSURE, dashboardData);
        redisCore.putHashMap(PlatformConstWords.REDIS_DASHBOARD_SERVER_PRESSURE, dashboardData, 500);
    }
}
