package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISysLogService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogException;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogOperate;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogSystem;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogTaskTime;
import cn.estsh.i3plus.pojo.platform.platrepositorymongo.SysLogExceptionRepository;
import cn.estsh.i3plus.pojo.platform.platrepositorymongo.SysLogOperateRepository;
import cn.estsh.i3plus.pojo.platform.platrepositorymongo.SysLogSystemRepository;
import cn.estsh.i3plus.pojo.platform.platrepositorymongo.SysLogTaskTimeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreBsonPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppSwitchUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import com.mongodb.client.AggregateIterable;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.model.Filters;
import io.swagger.annotations.ApiOperation;
import org.bson.Document;
import org.bson.conversions.Bson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.mongodb.MongoDbFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-2-13 下午4:16
 * @Modify:
 **/
@Service
public class SysLogService implements ISysLogService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysLogService.class);

    private SysLogTaskTimeRepository logTaskTimeRDao;
    private SysLogSystemRepository logSystemRDao;
    private SysLogOperateRepository logOperateRDao;
    private SysLogExceptionRepository logExceptionRDao;
    private MongoDbFactory mongoDbFactory;


    public SysLogTaskTimeRepository getSysLogTaskTimeRepository(){
        if(logTaskTimeRDao == null && ImppSwitchUtil.propSwitchMongo()){
            logTaskTimeRDao = SpringContextsUtil.getBean(SysLogTaskTimeRepository.class);
        }

        if(logTaskTimeRDao == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_MONGO_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return logTaskTimeRDao;
    }

    public SysLogSystemRepository getSysLogSystemRepository(){
        if(logSystemRDao == null && ImppSwitchUtil.propSwitchMongo()){
            logSystemRDao = SpringContextsUtil.getBean(SysLogSystemRepository.class);
        }

        if(logSystemRDao == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_MONGO_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return logSystemRDao;
    }

    public SysLogOperateRepository getSysLogOperateRepository(){
        if(logOperateRDao == null && ImppSwitchUtil.propSwitchMongo()){
            logOperateRDao = SpringContextsUtil.getBean(SysLogOperateRepository.class);
        }

        if(logOperateRDao == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_MONGO_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return logOperateRDao;
    }

    public SysLogExceptionRepository getSysLogExceptionRepository(){
        if(logExceptionRDao == null && ImppSwitchUtil.propSwitchMongo()){
            logExceptionRDao = SpringContextsUtil.getBean(SysLogExceptionRepository.class);
        }

        if(logExceptionRDao == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_MONGO_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return logExceptionRDao;
    }

    public MongoDbFactory getMongoDbFactory(){
        if(mongoDbFactory == null && ImppSwitchUtil.propSwitchMongo()){
            mongoDbFactory = SpringContextsUtil.getBean(MongoDbFactory.class);
        }

        if(mongoDbFactory == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_MONGO_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return mongoDbFactory;
    }



    /********************************************************* 系统异常日志 Start *********************************************************/

    @Override
    @ApiOperation(value = "系统异常日志复杂查询，分页，排序")
    public ListPager<SysLogException> querySysLogExceptionByPager(SysLogException sysLogException, Pager pager) {
        if(sysLogException == null) {
            pager = PagerHelper.getPager(pager, (int) getSysLogExceptionRepository().count());
            return new ListPager(getSysLogExceptionRepository().findAll(),pager);
        }else{
            Bson bson = CoreBsonPack.packBsonBySysLogException(sysLogException);
            pager = PagerHelper.getPager(pager,getSysLogExceptionRepository().findByBsonCount(bson));
            //分页，排序
            return new ListPager(getSysLogExceptionRepository().findByBsonPager(bson,pager,sysLogException.getOrderByParam()
                    ,sysLogException.getAscOrDesc()),pager);
        }
    }

    @Override
    public long deleteSysLogExceptionByDateTimeAfter(String dateTime) {
        return getSysLogExceptionRepository().deleteByProperty(CoreBsonPack.packBsonByDateTimeAfter(dateTime));
    }

    /******************************************************* 系统异常日志 End *********************************************************/


    /********************************************************* 操作日志 Start *********************************************************/

    @Override
    @ApiOperation(value = "添加操作日志",notes = "添加操作日志")
    public SysLogOperate insertSysLogOperate(SysLogOperate logOperate) {
        return getSysLogOperateRepository().insert(logOperate);
    }

    @Override
    @ApiOperation(value = "根据id删除日志",notes = "根据id删除日志")
    public void deleteSysLogOperateById(Long id) {
        getSysLogOperateRepository().deleteById(id);
    }

    @Override
    @ApiOperation(value = "查询全部操作日志",notes = "查询全部操作日志")
    public List listSysLogOperate() {
        return getSysLogOperateRepository().listPager(null);
    }

    @Override
    public void deleteSysLogOperateByIds(Long[] ids) {
        for(Long id : ids){
            getSysLogOperateRepository().deleteById(id);
        }
    }

    @Override
    @ApiOperation(value = "根据id查询操作日志",notes = "根据id查询操作日志")
    public SysLogOperate getSysLogOperateById(Long id) {
        return getSysLogOperateRepository().getById(id);
    }

    @Override
    @ApiOperation(value = "根据操作人姓名模糊查询日志",notes = "根据操作人姓名模糊查询日志")
    public List<SysLogOperate> listSysLogOperateByOperateUserName(String operateUserName) {
        return getSysLogOperateRepository().findByProperty("operateUserName",operateUserName);
    }

    @Override
    @ApiOperation(value = "查询所有日志数量")
    public long countAllSysLogOperate() {
        return getSysLogOperateRepository().listCount();
    }

    @Override
    @ApiOperation(value = "操作日志复杂查询，分页，排序",notes = "操作日志复杂查询，分页，排序")
    public ListPager<SysLogOperate> querySysLogOperateByPager(SysLogOperate logOperate, Pager pager) {
        if(logOperate == null) {
            pager = PagerHelper.getPager(pager, (int) getSysLogOperateRepository().count());
            return new ListPager(getSysLogOperateRepository().findAll(),pager);
        }else{
            Bson bson = CoreBsonPack.packBsonByLogOperate(logOperate);
            pager = PagerHelper.getPager(pager,getSysLogOperateRepository().findByBsonCount(bson));
            //分页，排序
            return new ListPager(getSysLogOperateRepository().findByBsonPager(bson,pager,logOperate.getOrderByParam(),logOperate.getAscOrDesc()),pager);
        }
    }

    @Override
    public List<SysLogOperate> findNewSysLogOperate(Integer num) {
        Pager page = new Pager();
        page.setCurrentPage(1);
        page.setPageSize(num);
        SysLogOperate sysLogOperate = new SysLogOperate();
        sysLogOperate.setOrderByParam("createDatetime");
        sysLogOperate.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.DESC.getValue());
        return getSysLogOperateRepository().findByBsonPager(null,page,sysLogOperate.getOrderByParam(),sysLogOperate.getAscOrDesc());
    }

    @Override
    public long deleteSysLogOperateByDateTimeAfter(String dateTime) {
        return getSysLogOperateRepository().deleteByProperty(CoreBsonPack.packBsonByDateTimeAfter(dateTime));
    }

    @Override
    public long countSysLogOperate() {
        return getSysLogOperateRepository().listCount();
    }

    /******************************************************* 操作日志 End *********************************************************/



    /******************************************************* 系统日志 Start *********************************************************/

    @Override
    @ApiOperation(value = "添加系统日志",notes = "添加系统日志")
    public void insertSysLogSystem(SysLogSystem logSystem) {
        getSysLogSystemRepository().insert(logSystem);
    }


    @Override
    public void insertSysLogBatch(int num,int waitTime){
        long t1 = System.currentTimeMillis();
        SysLogSystem sysLogSystem = null;
        for (int i = 0; i < num * 1000; i++) {
            sysLogSystem = new SysLogSystem();
            sysLogSystem.setLogClass("新增测试class");
            sysLogSystem.setLogMethod("新增测试method" +System.currentTimeMillis());
            sysLogSystem.setLogArgs("新增测试参数" + System.currentTimeMillis());
            sysLogSystem.setExecuteTime(0L);
            sysLogSystem.setArgsCount(1);
            sysLogSystem.setLogDetail("2333");
            sysLogSystem.setLogLevel(ImppEnumUtil.LOG_LEVEL.DEBUG.getValue());
            sysLogSystem.setLogModuleId(CommonEnumUtil.SOFT_TYPE.CORE.getValue());
            sysLogSystem.setLogTitle("新增测试");

            getSysLogSystemRepository().insert(sysLogSystem);
            try {
                this.wait(waitTime);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            LOGGER.info("插入SysLog耗时：" + (System.currentTimeMillis() - t1) + ",idx：" + i);
        }
    }

    @Override
    @ApiOperation(value = "删除日志",notes = "删除日志")
    public long deleteSysLogSystemById(Long id) {
        Bson bson = Filters.eq("id",id);
        return getSysLogSystemRepository().deleteByProperty(bson);
    }

    @Override
    @ApiOperation(value = "查询全部日志",notes = "查询全部日志")
    public List<SysLogSystem> listSysLogSystem() {
        return getSysLogSystemRepository().findAll();
    }

    @Override
    @ApiOperation(value = "根据id查询系统日志",notes = "根据id查询系统日志")
    public SysLogSystem getSysLogSystemById(long id) {
        return getSysLogSystemRepository().getById(id);
    }

    @Override
    @ApiOperation(value = "系统日志分页复杂查询",notes = "系统日志分页复杂查询")
    public ListPager querySysLogSystemByPager(SysLogSystem logSystem, Pager pager) {
        if (logSystem == null) {
            pager = PagerHelper.getPager(pager,getSysLogSystemRepository().listCount());
            return new ListPager(getSysLogSystemRepository().listPager(pager),pager);
        } else {
            Bson bson = CoreBsonPack.packBsonByLogSystem(logSystem);
            pager = PagerHelper.getPager(pager,getSysLogSystemRepository().findByBsonCount(bson));
            return new ListPager(getSysLogSystemRepository().findByBsonPager(bson,pager,logSystem.getOrderByParam()
                                    ,logSystem.getAscOrDesc()),pager);
        }
    }

    @Override
    @ApiOperation(value = "系统日志批量删除",notes = "系统日志批量删除")
    public long deleteSysLogSystemByIds(Long[] ids) {
        return getSysLogSystemRepository().deleteByIds(ids);
    }

    @Override
    @ApiOperation(value = "系统日志按id区间查询",notes = "系统日志按id区间查询")
    public List<SysLogSystem> findSysLogSystemByIdInterval(long min, long max) {
        return getSysLogSystemRepository().findByBson(CoreBsonPack.packBsonByInterval("_id",min,max));
    }

    @Override
    public long deleteSysLogSystemByDateTimeAfter(String dateTime) {
        return getSysLogSystemRepository().deleteByProperty(CoreBsonPack.packBsonByDateTimeAfter(dateTime));
    }


    /******************************************************* 系统日志 End *********************************************************/


    /******************************************************* 定时任务日志 Start *********************************************************/

    @Override
    @ApiOperation(value = "添加定时任务日志",notes = "添加定时任务日志")
    public void insertSysLogTaskTime(SysLogTaskTime sysLogTaskTime) {
        getSysLogTaskTimeRepository().insert(sysLogTaskTime);
    }

    @Override
    @ApiOperation(value = "删除定时任务日志",notes = "删除定时任务日志")
    public void deleteSysLogTaskTimeById(Long id) {
        getSysLogTaskTimeRepository().deleteById(id);
    }

    @Override
    @ApiOperation(value = "根据id查询定时任务日志",notes = "根据id查询定时任务日志")
    public SysLogTaskTime getSysLogTaskTimeById(Long id) {
        return getSysLogTaskTimeRepository().getById(id);
    }

    @Override
    @ApiOperation(value = "定时任务日志分页复杂查询",notes = "定时任务日志分页复杂查询")
    public ListPager querySysLogTaskTimeByPager(SysLogTaskTime sysLogTaskTime, Pager pager) {
        if (sysLogTaskTime == null) {
            pager = PagerHelper.getPager(pager,getSysLogTaskTimeRepository().listCount());
            return new ListPager(getSysLogTaskTimeRepository().listPager(pager),pager);
        } else {
            Bson bson = CoreBsonPack.packBsonByLogTaskTime(sysLogTaskTime);
            pager = PagerHelper.getPager(pager,getSysLogTaskTimeRepository().findByBsonCount(bson));
            return new ListPager(getSysLogTaskTimeRepository().findByBsonPager(bson,pager,sysLogTaskTime.getOrderByParam()
                    ,sysLogTaskTime.getAscOrDesc()),pager);
        }
    }

    @Override
    public long deleteSysLogTaskTimeByDateTimeAfter(String dateTime) {
        return getSysLogTaskTimeRepository().deleteByProperty(CoreBsonPack.packBsonByDateTimeAfter(dateTime));
    }
    /******************************************************* 定时任务日志 End *********************************************************/


    /********************************************************* 操作日志 Start *********************************************************/

    @Override
    public Map<String, Object> querySysLogSystemAvgExecuteTime(String startTime, String endTime) {
        // 筛选
        Document sub_match = new Document();
        sub_match.put("createDatetime", new Document("$gte", startTime).append("$lte", endTime));
       // 分组
        Document sub_group = new Document();
        sub_group.put("_id", "$createDatetime");
        sub_group.put("avg", new Document("$avg", "$executeTime"));


        Document match = new Document("$match", sub_match);
        Document group = new Document("$group", sub_group);
        Document sort = new Document("$sort", new Document("_id", -1));

        List<Document> aggregateList = new ArrayList<>();
        aggregateList.add(match);
        aggregateList.add(group);
        aggregateList.add(sort);

        AggregateIterable<Document> findIter = getMongoDbFactory().getDb()
                .getCollection(StringTool.toLowerCaseFirstOne(SysLogSystem.class.getSimpleName()))
                .aggregate(aggregateList);
        MongoCursor<Document> cursor = findIter.iterator();

        Map resultData = new LinkedHashMap();
        Document item_doc;
        while (cursor.hasNext()) {
            item_doc = cursor.next();
            resultData.put(item_doc.getString("_id"),MathOperation.round(item_doc.getDouble("avg"),0));
        }
        return resultData;
    }

    @Override
    public Map<String,Object> querySysLogSystemByLevel(String startTime, String endTime){
        // 筛选
        Document sub_match = new Document();
        sub_match.put("createDatetime", new Document("$gte", startTime).append("$lte", endTime));
        // 分组
        Document sub_group = new Document();
        sub_group.put("_id", "$logLevel");
        sub_group.put("count", new Document("$sum", 1));

        Document match = new Document("$match", sub_match);
        Document group = new Document("$group", sub_group);

        List<Document> aggregateList = new ArrayList<>();
        aggregateList.add(match);
        aggregateList.add(group);

        AggregateIterable<Document> findIter = getMongoDbFactory().getDb()
                .getCollection(StringTool.toLowerCaseFirstOne(SysLogSystem.class.getSimpleName()))
                .aggregate(aggregateList);
        MongoCursor<Document> cursor = findIter.iterator();

        Map resultData = new LinkedHashMap();
        Document item_doc;
        while (cursor.hasNext()) {
            item_doc = cursor.next();
            resultData.put(ImppEnumUtil.LOG_LEVEL.valueOfName(item_doc.getInteger("_id")),MathOperation.round(item_doc.getInteger("count"),0));
        }

        return resultData;
    }

    /******************************************************* 系统异常日志 End *********************************************************/


}
