package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleResourceService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.core.api.iservice.busi.ISystemResourceService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.SysConfigRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysDictionaryRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleLanguageRepository;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.RedisUtilTool;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.RESOURCE_PREFIX_MENU;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2019-01-11 12:34
 * @Modify:
 **/
@Service
public class SystemInitService implements ISystemInitService {

    private static final Logger LOGGER = LoggerFactory.getLogger(SystemLoginService.class);

    @Autowired
    private SysConfigRepository configRDao;

    @Autowired
    private SysDictionaryRepository dictionaryRDao;

    @Autowired
    public ISystemResourceService systemResourceService;

    @Autowired
    private ISysLocaleResourceService localeResourceService;

    @Autowired
    private SysLocaleLanguageRepository sysLocaleLanguageRDao;

    @Autowired
    private ISysMenuService menuService;

    @Resource(name= CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    private int type = ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue();

    @Override
    @ApiOperation(value = "初始化加载所有数据",notes = "初始化加载所有数据")
    public void putAndLoadAll() {
        type = ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue();
        putAndLoadSysConfig();
        putAndLoadSysDictionary();
        putAndLoadSysLocaleLanguage();
    }

    @Override
    @ApiOperation(value = "加载系统配置",notes = "加载系统配置")
    public void putAndLoadSysConfig() {
        List<SysConfig> list = configRDao.findAll();
        if(list != null && list.size() > 0){
            list.forEach(config -> {
                RedisUtilTool.setSysConfigByConfigCode(redisRes, config);
            });
        }
        LOGGER.info("加载系统配置数量：【{}】",list.size());
    }

    @Override
    @ApiOperation(value = "加载字典数据",notes = "加载字典数据")
    public void putAndLoadSysDictionary() {
        try {
            List<SysDictionary> list = dictionaryRDao.findAll();
            if(list != null && list.size() > 0){
                list.stream().filter(sysDictionary-> sysDictionary.getParentCodeRdd() != null).forEach(System.out :: print);
                Map<String, List<SysDictionary>> parentCodeMap = list.stream().filter(sysDictionary-> sysDictionary.getParentCodeRdd() != null).collect(Collectors.groupingBy(SysDictionary::getParentCodeRdd));
                Map<Long, List<SysDictionary>> parentIdMap = list.stream().collect(Collectors.groupingBy(SysDictionary::getParentId));

                for (String key : parentCodeMap.keySet()) {
                    //存放于缓存
                    RedisUtilTool.setSysDictionaryListByCode(redisRes, key, parentCodeMap.get(key));
                }

                for (long key : parentIdMap.keySet()) {
                    putDataToCache(CommonConstWords.REDIS_PREFIX_CACHE_DICTIONARY + ":" + key, parentIdMap.get(key),Object.class);
                    RedisUtilTool.setSysDictionaryListByParentId(redisRes,key,parentIdMap.get(key));
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    @ApiOperation(value = "加载语言数据",notes = "加载语言数据")
    public void putAndLoadSysLocaleLanguage() {
        // Redis 缓存
        if (type == ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue()) {
            /**
             * 整体数据结构：
             * code <langCode : value>
             *    先根据code查询，判断是否存在，然后通过语言获取对应的值
             *
             * 语言代码 hashMap结构
             * langCode < code : value>
             *
             * 语言代码+模块 hashMap结构
             * langCode + softType < code : value>
             *
             */
            HashMap<String, HashMap<String, String>> resMap = new HashMap();
            HashMap<String, String> lanMap = null;
            HashMap<String, HashMap<String, String>> webLangMap = new HashMap<>();
            HashMap<String, String> webResMap;
            HashMap<String, HashMap<String, String>> webSoftLangMap = new HashMap<>();
            HashMap<String, String> webSoftResMap;
            String softResKey = null;
            Integer resCount = systemResourceService.getSysLocaleResourceCount(null);

            //语言数量
            List<SysLocaleLanguage> langList = systemResourceService.listSysLocaleLanguage(null);

            LOGGER.info("【加载平台资源】共有{}个资源，{}种语言。", resCount, langList.size());
            Pager pager = new Pager(resCount, 5000);
            pager.setCurrentPage(0);
            //查询所有资源
            ListPager<SysLocaleResource> resourceList;
            do {
                pager.next();
                resourceList = systemResourceService.listSysLocaleResourceByPager(null, pager);
                resMap = new HashMap();
                webLangMap = new HashMap<>();
                webSoftLangMap = new HashMap<>();

                for (SysLocaleResource res : resourceList.getObjectList()) {
                    lanMap = resMap.get(res.getResourceKey());
                    //判断是否已存在资源信息
                    if (lanMap == null) {
                        lanMap = new HashMap();
                        //将key/代码放入
                        resMap.put(res.getResourceKey(), lanMap);
                    }
                    //根据语言放入
                    lanMap.put(res.getLanguageCode(), res.getResourceValue());

                    // web 资源
                    if (!webLangMap.containsKey(res.getLanguageCode())) {
                        webLangMap.put(res.getLanguageCode(), new HashMap<>());
                    }
                    // 放入资源信息
                    webResMap = webLangMap.get(res.getLanguageCode());
                    webResMap.put(res.getResourceKey(), res.getResourceValue());

                    // web 模块资源
                    softResKey = res.getLanguageCode() + ":" + res.getSoftTypeVal();
                    if (!webSoftLangMap.containsKey(softResKey)) {
                        webSoftLangMap.put(softResKey, new HashMap<>());
                    }
                    // 放入模块资源信息
                    webSoftResMap = webSoftLangMap.get(softResKey);
                    webSoftResMap.put(res.getResourceKey(), res.getResourceValue());
                }

                for (String key : resMap.keySet()) {
                    //放入缓存
                    if (resMap.get(key) != null) {
                        redisRes.putHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + key, resMap.get(key), 0);
                    }
                }

                for (String key : webLangMap.keySet()) {
                    //放入缓存
                    if (webLangMap.get(key) != null) {
                        redisRes.putHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE + ":" + key, webLangMap.get(key), 0);
                        redisRes.putHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + key, webLangMap.get(key), 0);
                    }
                }

                for (String key : webSoftLangMap.keySet()) {
                    //放入缓存
                    if (webSoftLangMap.get(key) != null) {
                        redisRes.putHashMap(CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_WEB + ":" + key, webSoftLangMap.get(key), 0);
                    }
                }

                LOGGER.info("【加载平台资源】加载中 {} / {} ", pager.getEndRow(), pager.getTotalRows());
            } while (pager.hasNext());


            LOGGER.info("【加载平台资源】加载完成。");
        }
        //加载语言数据
        putAndLoadSysMenu();
    }

    @ApiOperation(value = "加载语言数据",notes = "加载语言数据")
    public void putAndLoadSysMenu() {

        SysLocaleResource resource = new SysLocaleResource();
        resource.setResourceType(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.MENU.getValue());
        List<SysLocaleResource> resources = localeResourceService.findSysLocaleResource(resource);
        List<SysMenu> menus = menuService.findSysMenuAll();
        List<SysLocaleLanguage> languages = sysLocaleLanguageRDao.findAll();

        Map<String,SysLocaleResource> resourceKeyMap = new HashMap<>();
        List<SysLocaleResource> insertList = new ArrayList<>();
        Map<String,String> redisCacheMap = new HashMap<>();

        if(resources != null && resources.size() > 0){
            resources.forEach(res -> resourceKeyMap.put(StringUtils.join(res.getLanguageCode(),res.getResourceKey()),res));
        }
        String redisLanguageKey;
        SysLocaleResource res;
        if(menus != null && menus.size() > 0 && languages != null && languages.size() > 0){
            for (SysLocaleLanguage language : languages) {
                 redisLanguageKey = CommonConstWords.REDIS_PREFIX_CACHE_LANGUAGE_MENU + ":" + language.getLanguageCode();

                for (SysMenu menu : menus) {
                    // 多次校验
                    res = resourceKeyMap.get(StringUtils.join(language.getLanguageCode(),RESOURCE_PREFIX_MENU,menu.getName()));
                    if(res == null){
                        res = localeResourceService.getSysLocaleResourceByLanguageCodeAndResKey(language.getLanguageCode(),
                                RESOURCE_PREFIX_MENU + menu.getName());
                    }

                    if(res == null){
                        // 当系统没有相应的功能权限资源时创建资源
                        res = new SysLocaleResource();
                        res.setResourceType(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.MENU.getValue());
                        res.setLanguageCode(language.getLanguageCode());
                        res.setLanguageNameRdd(language.getLanguageName());
                        res.setResourceKey(RESOURCE_PREFIX_MENU + menu.getName());
                        res.setResourceValue(menu.getName());
                        res.setIsSystem(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                        ConvertBean.serviceModelInitialize(res, PlatformConstWords.SYSTEM_INIT_USER);

                        insertList.add(res);
                    }

                    resourceKeyMap.put(StringUtils.join(res.getLanguageCode(),res.getResourceKey()),res);
                    redisCacheMap.put(res.getResourceKey(),res.getResourceValue());
                }

                redisRes.putHashMap(redisLanguageKey,redisCacheMap,0);
                redisCacheMap.clear();
            }
        }

        if(insertList != null && insertList.size() > 0){
            localeResourceService.saveSysLocaleResource(insertList);
        }
    }

    /**
     * 缓存存放数据统一管理
     * @param key
     * @param data
     * @param dataType
     */
    private void putDataToCache(String key,Object data,Class dataType){
        if(type == ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue()){
            LOGGER.info(" Put Cache Redis Key:{},value:{},dataType:{}",key,data,dataType);
            // Redis 缓存
            if(dataType == List.class){
                redisRes.putObject(key,data,-1);
            }else if(dataType == Object.class){
                redisRes.putObject(key,data,-1);
            }else{
                LOGGER.error("不支持树形{}存放缓存！",dataType);
            }
        }
    }

    /**
     * 从缓存获取对象数据
     * @param key
     * @param dataType
     * @return
     */
    @Override
    @ApiOperation(value = "获取缓存中的数据",notes = "获取缓存中的数据")
    public Object getDataFromCache(String key,Class dataType){
        if(type == ImppEnumUtil.SYS_CACHE_TYPE.REDIS.getValue()){
            // Redis 缓存
            if(dataType == List.class){
                return redisRes.getList(key,0,-1);
            }else if(dataType == SysConfig.class){
                return redisRes.getObject(key);
            }else{
                LOGGER.error("不支持树形{}存放缓存！",dataType);
            }
        }
        return null;
    }
}
