package test.cn.estsh.i3plus.core.apiservice.util;

import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleResourceService;
import cn.estsh.i3plus.core.apiservice.serviceimpl.busi.TestBase;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.File;
import java.util.*;

/**
 * @Description :
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2019-11-25 20:28
 * @Modify:
 **/
public class ResourceUtil extends TestBase {

    @Autowired
    private ISysLocaleResourceService sysLocaleResourceService;

    public static void main(String[] args) {
        new ResourceUtil().initResource();
    }

    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Test
    public void initResource() {
        File file = new File("C:\\Users\\yunha\\Desktop\\资源文件.xlsx");
        try {
            Workbook wb = new XSSFWorkbook(file);
            // 前端资源
            Sheet webSheet = wb.getSheet("SUM");
            Row row = null;
            SysLocaleResource sysLocaleResource;
            Map<String, Map<String, SysLocaleResource>> resource = new HashMap<>();
            Map<String, SysLocaleResource> item = new HashMap<>();
            snowflakeIdMaker = SnowflakeIdMaker.getSnowflakeIdMaker(1L,1);
            for (int i = 1; i < webSheet.getLastRowNum(); i++) {
                row = webSheet.getRow(i);

                if (row == null || row.getCell(2) == null || row.getCell(3) == null) {
                    continue;
                }

//                System.out.println(snowflakeIdMaker.nextId() + "行:" + i);
                row.getCell(1).setCellType(CellType.STRING); // 资源类型
                row.getCell(2).setCellType(CellType.STRING); // 模块
                row.getCell(3).setCellType(CellType.STRING); // key
                row.getCell(4).setCellType(CellType.STRING); // 中文
                row.getCell(5).setCellType(CellType.STRING); // 英文

                int resType = CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.descOf(row.getCell(1).getStringCellValue());
                int softType = CommonEnumUtil.SOFT_TYPE.descOf(row.getCell(2).getStringCellValue());
                String key = row.getCell(3).getStringCellValue();
                String cnVal = row.getCell(4).getStringCellValue();
                String enVal = row.getCell(5).getStringCellValue();
                if (resource.containsKey(key)) {
                    item = resource.get(key);
                } else {
                    item = new HashMap<>();
                    resource.put(key,item);
                }
                //中文
                sysLocaleResource = new SysLocaleResource();
                sysLocaleResource.setId(snowflakeIdMaker.nextId());
                sysLocaleResource.setLanguageCode(YY.CN.getValue());
                sysLocaleResource.setLanguageNameRdd(YY.CN.getDescription());
                sysLocaleResource.setResourceType(resType);
                sysLocaleResource.setSoftType(softType);
                sysLocaleResource.setResourceKey(key);
                sysLocaleResource.setResourceValue(cnVal);
                ConvertBean.serviceModelInitialize(sysLocaleResource, PlatformConstWords.SYSTEM_INIT_USER);

                item.put(YY.CN.getValue(), sysLocaleResource);

//                System.out.println(JSON.toJSONString(sysLocaleResource));

                // 英文
                sysLocaleResource = new SysLocaleResource();
                sysLocaleResource.setId(snowflakeIdMaker.nextId());
                sysLocaleResource.setResourceType(resType);
                sysLocaleResource.setSoftType(softType);
                sysLocaleResource.setLanguageCode(YY.EN.getValue());
                sysLocaleResource.setLanguageNameRdd(YY.EN.getDescription());
                sysLocaleResource.setResourceType(resType);
                sysLocaleResource.setSoftType(softType);
                sysLocaleResource.setResourceKey(key);
                sysLocaleResource.setResourceValue(enVal);
                ConvertBean.serviceModelInitialize(sysLocaleResource, PlatformConstWords.SYSTEM_INIT_USER);

                item.put(YY.EN.getValue(), sysLocaleResource);

//                System.out.println(JSON.toJSONString(sysLocaleResource));
                System.out.format("UPDATE  sys_locale_resource resource set  resource.soft_type = %s where resource" +
                        ".resource_key = '%s';\n ",softType,key);
            }
            List<SysLocaleResource> resourceList = new ArrayList<>();
            for (String key : resource.keySet()) {
                for (SysLocaleResource value : resource.get(key).values()) {
                    resourceList.add(value);
                }
            }

            for (List<SysLocaleResource> sysLocaleResources : splitList(resourceList, 1000)) {
                sysLocaleResourceService.saveSysLocaleResource(sysLocaleResources);
            }


        } catch (Exception e) {
            e.printStackTrace();
        }

    }
    /**
     * 按指定大小，分隔集合，将集合按规定个数分为n个部分
     * @param <T>
     *
     * @param list
     * @param len
     * @return
     */
    public static <T> List<List<T>> splitList(List<T> list, int len) {

        if (list == null || list.isEmpty() || len < 1) {
            return Collections.emptyList();
        }

        List<List<T>> result = new ArrayList<>();

        int size = list.size();
        int count = (size + len - 1) / len;

        for (int i = 0; i < count; i++) {
            List<T> subList = list.subList(i * len, ((i + 1) * len > size ? size : len * (i + 1)));
            result.add(subList);
        }

        return result;
    }
    /**
     * 处理标识
     */
    public enum YY {
        CN("CN/zh", "简体中文"), EN("US/en", "English");

        private String value;
        private String description;

        private YY(String value, String description) {
            this.value = value;
            this.description = description;
        }

        public String getValue() {
            return value;
        }

        public String getDescription() {
            return description;
        }

    }

}
